@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2014-2019 Nils Christopher Brause, Philipp Kerling, Bernd Kuhls
+ *  Copyright (c) 2014-2022 Nils Christopher Brause, Philipp Kerling, Bernd Kuhls
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -216,16 +216,16 @@ struct request_t : public event_t
     if(!ret.summary.empty())
       ss << "      \\return " << ret.summary << std::endl;
     for(auto const& arg : args)
+    {
+      if(arg.type == "new_id")
       {
-        if(arg.type == "new_id")
-          {
-            if(arg.interface.empty())
-              ss << "      \\param interface Interface to bind" << std::endl
-                 << "      \\param version Interface version" << std::endl;
-          }
-        else
-          ss << "      \\param " << sanitise(arg.name) << " " << arg.summary << std::endl;
+        if(arg.interface.empty())
+          ss << "      \\param interface Interface to bind" << std::endl
+             << "      \\param version Interface version" << std::endl;
       }
+      else
+        ss << "      \\param " << sanitise(arg.name) << " " << arg.summary << std::endl;
+    }
     ss << description << std::endl
        << "  */" << std::endl;
 
@@ -237,10 +237,10 @@ struct request_t : public event_t
 
     for(auto const& arg : args)
       if(arg.type == "new_id")
-        {
-          if(arg.interface.empty())
-            ss << "proxy_t &interface, uint32_t version, ";
-        }
+      {
+        if(arg.interface.empty())
+          ss << "proxy_t &interface, uint32_t version, ";
+      }
       else
         ss << arg.print_argument() << ", ";
 
@@ -278,13 +278,13 @@ struct request_t : public event_t
     bool new_id_arg = false;
     for(auto const& arg : args)
       if(arg.type == "new_id")
+      {
+        if(arg.interface.empty())
         {
-          if(arg.interface.empty())
-            {
-              ss << "proxy_t &interface, uint32_t version, ";
-              new_id_arg = true;
-            }
+          ss << "proxy_t &interface, uint32_t version, ";
+          new_id_arg = true;
         }
+      }
       else
         ss << arg.print_argument() << ", ";
 
@@ -296,46 +296,46 @@ struct request_t : public event_t
     if(ret.name.empty())
       ss <<  "  marshal(" << opcode << "U, ";
     else if(ret.interface.empty())
-      {
-        ss << "  proxy_t p = marshal_constructor_versioned(" << opcode << "U, interface.interface, version, ";
-      }
+    {
+      ss << "  proxy_t p = marshal_constructor_versioned(" << opcode << "U, interface.interface, version, ";
+    }
     else
-      {
-        ss << "  proxy_t p = marshal_constructor(" << opcode << "U, &" << ret.interface << "_interface, ";
-      }
+    {
+      ss << "  proxy_t p = marshal_constructor(" << opcode << "U, &" << ret.interface << "_interface, ";
+    }
 
     for(auto const& arg : args)
+    {
+      if(arg.type == "new_id")
       {
-        if(arg.type == "new_id")
-          {
-            if(arg.interface.empty())
-              ss << "std::string(interface.interface->name), version, ";
-            ss << "nullptr, ";
-          }
-        else if(arg.type == "fd")
-          ss << "argument_t::fd(" << sanitise(arg.name) << "), ";
-        else if(arg.type == "object")
-          ss << sanitise(arg.name) << ".proxy_has_object() ? reinterpret_cast<wl_object*>(" << sanitise(arg.name) << ".c_ptr()) : nullptr, ";
-        else if(!arg.enum_name.empty())
-          ss << "static_cast<" << arg.print_enum_wire_type() << ">(" << sanitise(arg.name) + "), ";
-        else
-          ss << sanitise(arg.name) + ", ";
+        if(arg.interface.empty())
+          ss << "std::string(interface.interface->name), version, ";
+        ss << "nullptr, ";
       }
+      else if(arg.type == "fd")
+        ss << "argument_t::fd(" << sanitise(arg.name) << "), ";
+      else if(arg.type == "object")
+        ss << sanitise(arg.name) << ".proxy_has_object() ? reinterpret_cast<wl_object*>(" << sanitise(arg.name) << ".c_ptr()) : nullptr, ";
+      else if(!arg.enum_name.empty())
+        ss << "static_cast<" << arg.print_enum_wire_type() << ">(" << sanitise(arg.name) + "), ";
+      else
+        ss << sanitise(arg.name) + ", ";
+    }
 
     ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
     ss << ");" << std::endl;
 
     if(!ret.name.empty())
+    {
+      if(new_id_arg)
       {
-        if(new_id_arg)
-          {
-            ss << "  interface = interface.copy_constructor(p);" << std::endl
-               << "  return interface;" << std::endl;
-          }
-        else
-          ss << "  return " << ret.print_type() << "(p);" << std::endl;
+        ss << "  interface = interface.copy_constructor(p);" << std::endl
+           << "  return interface;" << std::endl;
       }
+      else
+        ss << "  return " << ret.print_type() << "(p);" << std::endl;
+    }
     ss << "}";
 
     if(!availability_function_name().empty())
@@ -392,22 +392,22 @@ struct enumeration_t : public element_t
          << "    : detail::bitfield<" << width << ", " << id << ">(value) {}" << std::endl;
 
     for(auto const& entry : entries)
-      {
-        if(!entry.summary.empty())
-          ss << "  /** \\brief " << entry.summary << " */" << std::endl;
+    {
+      if(!entry.summary.empty())
+        ss << "  /** \\brief " << entry.summary << " */" << std::endl;
 
-        if(!bitfield)
-          ss << "  " << sanitise(entry.name) << " = " << entry.value << "," << std::endl;
-        else
-          ss << "  static const detail::bitfield<" << width << ", " << id << "> " << sanitise(entry.name) << ";" << std::endl;
-      }
+      if(!bitfield)
+        ss << "  " << sanitise(entry.name) << " = " << entry.value << "," << std::endl;
+      else
+        ss << "  static const detail::bitfield<" << width << ", " << id << "> " << sanitise(entry.name) << ";" << std::endl;
+    }
 
     if(!bitfield)
-      {
-        ss.str(ss.str().substr(0, ss.str().size()-2));
-        ss.seekp(0, std::ios_base::end);
-        ss << std::endl;
-      }
+    {
+      ss.str(ss.str().substr(0, ss.str().size()-2));
+      ss.seekp(0, std::ios_base::end);
+      ss << std::endl;
+    }
 
     ss << "};" << std::endl;
     return ss.str();
@@ -418,10 +418,10 @@ struct enumeration_t : public element_t
     std::stringstream ss;
     if(bitfield)
       for(auto const& entry : entries)
-        {
-          ss << "const bitfield<" << width << ", " << id << "> " << iface_name << "_" << name
-             << "::" << sanitise(entry.name) << "{" << entry.value << "};" << std::endl;
-        }
+      {
+        ss << "const bitfield<" << width << ", " << id << "> " << iface_name << "_" << name
+           << "::" << sanitise(entry.name) << "{" << entry.value << "};" << std::endl;
+      }
     return ss.str();
   }
 };
@@ -575,17 +575,17 @@ struct interface_t : public element_t
        << "{" << std::endl;
 
     if(!events.empty())
-      {
-        ss << "  std::shared_ptr<events_t> events = std::static_pointer_cast<events_t>(e);" << std::endl
-           << "  switch(opcode)" << std::endl
-           << "    {" << std::endl;
+    {
+      ss << "  std::shared_ptr<events_t> events = std::static_pointer_cast<events_t>(e);" << std::endl
+         << "  switch(opcode)" << std::endl
+         << "    {" << std::endl;
 
-        int opcode = 0;
-        for(auto const& event : events)
-          ss << event.print_dispatcher(opcode++) << std::endl;
+      int opcode = 0;
+      for(auto const& event : events)
+        ss << event.print_dispatcher(opcode++) << std::endl;
 
-        ss << "    }" << std::endl;
-      }
+      ss << "    }" << std::endl;
+    }
 
     ss << "  return 0;" << std::endl
        << "}" << std::endl;
@@ -600,69 +600,69 @@ struct interface_t : public element_t
   {
     std::stringstream ss;
     for(auto const& request : requests)
-      {
-        ss << "const wl_interface* " << name << "_interface_" << request.name << "_request[" << request.args.size() << "] = {" << std::endl;
-        for(auto const& arg : request.args)
-          if(!arg.interface.empty())
-            ss  << "  &" << arg.interface << "_interface," << std::endl;
-          else
-            ss  << "  nullptr," << std::endl;
-        ss << "};" << std::endl
-           << std::endl;
-      }
+    {
+      ss << "const wl_interface* " << name << "_interface_" << request.name << "_request[" << request.args.size() << "] = {" << std::endl;
+      for(auto const& arg : request.args)
+        if(!arg.interface.empty())
+          ss  << "  &" << arg.interface << "_interface," << std::endl;
+        else
+          ss  << "  nullptr," << std::endl;
+      ss << "};" << std::endl
+         << std::endl;
+    }
     for(auto const& event : events)
-      {
-        ss << "const wl_interface* " << name << "_interface_" << event.name << "_event[" << event.args.size() << "] = {" << std::endl;
-        for(auto const& arg : event.args)
-          if(!arg.interface.empty())
-            ss  << "  &" << arg.interface << "_interface," << std::endl;
-          else
-            ss  << "  nullptr," << std::endl;
-        ss << "};" << std::endl
-           << std::endl;
-      }
+    {
+      ss << "const wl_interface* " << name << "_interface_" << event.name << "_event[" << event.args.size() << "] = {" << std::endl;
+      for(auto const& arg : event.args)
+        if(!arg.interface.empty())
+          ss  << "  &" << arg.interface << "_interface," << std::endl;
+        else
+          ss  << "  nullptr," << std::endl;
+      ss << "};" << std::endl
+         << std::endl;
+    }
     ss << "const wl_message " << name << "_interface_requests[" << requests.size() << "] = {" << std::endl;
     for(auto const& request : requests)
+    {
+      ss << "  {" << std::endl
+         << "    \"" << request.name << "\"," << std::endl
+         << "    \"";
+      if(request.since > 1)
+        ss << request.since;
+      for(auto const& arg : request.args)
       {
-        ss << "  {" << std::endl
-           << "    \"" << request.name << "\"," << std::endl
-           << "    \"";
-        if(request.since > 1)
-          ss << request.since;
-        for(auto const& arg : request.args)
-          {
-            if(arg.allow_null)
-              ss << "?";
-            if(arg.type == "new_id" && arg.interface.empty())
-              ss << "su";
-            ss << arg.print_short();
-          }
-        ss << "\"," << std::endl
-           << "    " << name << "_interface_" << request.name << "_request," << std::endl
-           << "  }," << std::endl;
+        if(arg.allow_null)
+          ss << "?";
+        if(arg.type == "new_id" && arg.interface.empty())
+          ss << "su";
+        ss << arg.print_short();
       }
+      ss << "\"," << std::endl
+         << "    " << name << "_interface_" << request.name << "_request," << std::endl
+         << "  }," << std::endl;
+    }
     ss << "};" << std::endl
        << std::endl;
     ss << "const wl_message " << name << "_interface_events[" << events.size() << "] = {" << std::endl;
     for(auto const& event : events)
+    {
+      ss << "  {" << std::endl
+         << "    \"" << event.name << "\"," << std::endl
+         << "    \"";
+      if(event.since > 1)
+        ss << event.since;
+      for(auto const& arg : event.args)
       {
-        ss << "  {" << std::endl
-           << "    \"" << event.name << "\"," << std::endl
-           << "    \"";
-        if(event.since > 1)
-          ss << event.since;
-        for(auto const& arg : event.args)
-          {
-            if(arg.allow_null)
-              ss << "?";
-            if(arg.type == "new_id" && arg.interface.empty())
-              ss << "su";
-            ss << arg.print_short();
-          }
-        ss << "\"," << std::endl
-           << "    " << name << "_interface_" << event.name << "_event," << std::endl
-           << "  }," << std::endl;
+        if(arg.allow_null)
+          ss << "?";
+        if(arg.type == "new_id" && arg.interface.empty())
+          ss << "su";
+        ss << arg.print_short();
       }
+      ss << "\"," << std::endl
+         << "    " << name << "_interface_" << event.name << "_event," << std::endl
+         << "  }," << std::endl;
+    }
     ss << "};" << std::endl
        << std::endl;
     ss << "const wl_interface wayland::detail::" << name << "_interface =" << std::endl
@@ -684,11 +684,11 @@ std::string unprefix(const std::string &name)
 {
   auto prefix_len = name.find('_');
   if(prefix_len != std::string::npos)
-    {
-      auto prefix = name.substr(0, prefix_len);
-      if(prefix == "wl" || prefix == "wp")
-        return name.substr(prefix_len+1, name.size());
-    }
+  {
+    auto prefix = name.substr(0, prefix_len);
+    if(prefix == "wl" || prefix == "wp")
+      return name.substr(prefix_len+1, name.size());
+  }
   return name;
 }
 
@@ -711,8 +711,8 @@ void parse_args(int argc, char **argv, std::vector<arg_t>& map, std::vector<std:
     else
     {
       std::string value;
-      if (c + 1 < argc && argv[c+1][0] != '-')
-	value = argv[++c];
+      if(c + 1 < argc && argv[c+1][0] != '-')
+        value = argv[++c];
       map.push_back(arg_t{str.substr(1), value});
     }
   }
@@ -725,195 +725,195 @@ int main(int argc, char *argv[])
   parse_args(argc, argv, map, extra);
 
   if(extra.size() < 3)
-    {
-      std::cerr << "Usage:" << std::endl
-                << "  " << argv[0] << " [-x extra_header.hpp] protocol1.xml [protocol2.xml ...] protocol.hpp protocol.cpp" << std::endl;
-      return 1;
-    }
+  {
+    std::cerr << "Usage:" << std::endl
+              << "  " << argv[0] << " [-x extra_header.hpp] protocol1.xml [protocol2.xml ...] protocol.hpp protocol.cpp" << std::endl;
+    return 1;
+  }
 
   std::list<interface_t> interfaces;
   int enum_id = 0;
 
   for(int c = 0; c < extra.size()-2; c++)
+  {
+    xml_document doc;
+    doc.load_file(extra[c].c_str());
+    auto protocol = doc.child("protocol");
+
+    for(auto const& interface : protocol.children("interface"))
     {
-      xml_document doc;
-      doc.load_file(extra[c].c_str());
-      auto protocol = doc.child("protocol");
+      interface_t iface;
+      iface.destroy_opcode = -1;
+      iface.orig_name = interface.attribute("name").value();
+      iface.name = unprefix(iface.orig_name);
+      if(interface.attribute("version"))
+        iface.version = std::stoi(std::string(interface.attribute("version").value()), nullptr, 0);
+      else
+        iface.version = 1;
+      if(interface.child("description"))
+      {
+        auto description = interface.child("description");
+        iface.summary = description.attribute("summary").value();
+        iface.description = description.text().get();
+      }
 
-      for(auto const& interface : protocol.children("interface"))
+      interface_names.push_back(iface.name);
+
+      int opcode = 0; // Opcodes are in order of the XML. (Sadly undocumented)
+      for(auto const& request : interface.children("request"))
+      {
+        request_t req;
+        req.opcode = opcode++;
+        req.name = request.attribute("name").value();
+
+        if(request.attribute("since"))
+          req.since = std::stoi(std::string(request.attribute("since").value()), nullptr, 0);
+        else
+          req.since = 1;
+
+        if(request.child("description"))
         {
-          interface_t iface;
-          iface.destroy_opcode = -1;
-          iface.orig_name = interface.attribute("name").value();
-          iface.name = unprefix(iface.orig_name);
-          if(interface.attribute("version"))
-            iface.version = std::stoi(std::string(interface.attribute("version").value()), nullptr, 0);
-          else
-            iface.version = 1;
-          if(interface.child("description"))
-            {
-              auto description = interface.child("description");
-              iface.summary = description.attribute("summary").value();
-              iface.description = description.text().get();
-            }
-
-          interface_names.push_back(iface.name);
-
-          int opcode = 0; // Opcodes are in order of the XML. (Sadly undocumented)
-          for(auto const& request : interface.children("request"))
-            {
-              request_t req;
-              req.opcode = opcode++;
-              req.name = request.attribute("name").value();
-
-              if(request.attribute("since"))
-                req.since = std::stoi(std::string(request.attribute("since").value()), nullptr, 0);
-              else
-                req.since = 1;
-
-              if(request.child("description"))
-                {
-                  auto description = request.child("description");
-                  req.summary = description.attribute("summary").value();
-                  req.description = description.text().get();
-                }
-
-              // destruction takes place through the class destuctor
-              if(req.name == "destroy")
-                iface.destroy_opcode = req.opcode;
-              for(auto const& argument : request.children("arg"))
-                {
-                  argument_t arg;
-                  arg.type = argument.attribute("type").value();
-                  arg.name = argument.attribute("name").value();
-
-                  if(argument.attribute("summary"))
-                    arg.summary = argument.attribute("summary").value();
-
-                  if(argument.attribute("interface"))
-                    arg.interface = unprefix(argument.attribute("interface").value());
-
-                  if(argument.attribute("enum"))
-                    {
-                      std::string tmp = argument.attribute("enum").value();
-                      if(tmp.find('.') == std::string::npos)
-                        {
-                          arg.enum_iface = iface.name;
-                          arg.enum_name = tmp;
-                        }
-                      else
-                        {
-                          arg.enum_iface = unprefix(tmp.substr(0, tmp.find('.')));
-                          arg.enum_name = tmp.substr(tmp.find('.')+1);
-                        }
-                    }
-
-                  arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
-
-                  if(arg.type == "new_id")
-                    req.ret = arg;
-                  req.args.push_back(arg);
-                }
-              iface.requests.push_back(req);
-            }
-
-          for(auto const& event : interface.children("event"))
-            {
-              event_t ev;
-              ev.name = event.attribute("name").value();
-
-              if(event.attribute("since"))
-                ev.since = std::stoi(std::string(event.attribute("since").value()), nullptr, 0);
-              else
-                ev.since = 1;
-
-              if(event.child("description"))
-                {
-                  auto description = event.child("description");
-                  ev.summary = description.attribute("summary").value();
-                  ev.description = description.text().get();
-                }
-
-              for(auto const& argument : event.children("arg"))
-                {
-                  argument_t arg;
-                  arg.type = argument.attribute("type").value();
-                  arg.name = argument.attribute("name").value();
-
-                  if(argument.attribute("summary"))
-                    arg.summary = argument.attribute("summary").value();
-
-                  if(argument.attribute("interface"))
-                    arg.interface = unprefix(argument.attribute("interface").value());
-
-                  if(argument.attribute("enum"))
-                    {
-                      std::string tmp = argument.attribute("enum").value();
-                      if(tmp.find('.') == std::string::npos)
-                        {
-                          arg.enum_iface = iface.name;
-                          arg.enum_name = tmp;
-                        }
-                      else
-                        {
-                          arg.enum_iface = unprefix(tmp.substr(0, tmp.find('.')));
-                          arg.enum_name = tmp.substr(tmp.find('.')+1);
-                        }
-                    }
-
-                  arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
-
-                  ev.args.push_back(arg);
-                }
-              iface.events.push_back(ev);
-            }
-
-          for(auto const& enumeration : interface.children("enum"))
-            {
-              enumeration_t enu;
-              enu.name = enumeration.attribute("name").value();
-              if(enumeration.child("description"))
-                {
-                  auto description = enumeration.child("description");
-                  enu.summary = description.attribute("summary").value();
-                  enu.description = description.text().get();
-                }
-
-              if(enumeration.attribute("bitfield"))
-                {
-                  std::string tmp = enumeration.attribute("bitfield").value();
-                  enu.bitfield = (tmp == "true");
-                }
-              else
-                enu.bitfield = false;
-              enu.id = enum_id++;
-              enu.width = 0;
-
-              for(auto entry = enumeration.child("entry"); entry;
-                  entry = entry.next_sibling("entry"))
-                {
-                  enum_entry_t enum_entry;
-                  enum_entry.name = entry.attribute("name").value();
-                  if(enum_entry.name == "default"
-                     || isdigit(enum_entry.name.at(0)))
-                    enum_entry.name.insert(0, 1, '_');
-                  enum_entry.value = entry.attribute("value").value();
-
-                  if(entry.attribute("summary"))
-                    enum_entry.summary = entry.attribute("summary").value();
-
-                  auto tmp = static_cast<uint32_t>(std::log2(stol(enum_entry.value, nullptr, 0))) + 1U;
-                  if(tmp > enu.width)
-                    enu.width = tmp;
-
-                  enu.entries.push_back(enum_entry);
-                }
-              iface.enums.push_back(enu);
-            }
-
-          interfaces.push_back(iface);
+          auto description = request.child("description");
+          req.summary = description.attribute("summary").value();
+          req.description = description.text().get();
         }
+
+        // destruction takes place through the class destuctor
+        if(req.name == "destroy")
+          iface.destroy_opcode = req.opcode;
+        for(auto const& argument : request.children("arg"))
+        {
+          argument_t arg;
+          arg.type = argument.attribute("type").value();
+          arg.name = argument.attribute("name").value();
+
+          if(argument.attribute("summary"))
+            arg.summary = argument.attribute("summary").value();
+
+          if(argument.attribute("interface"))
+            arg.interface = unprefix(argument.attribute("interface").value());
+
+          if(argument.attribute("enum"))
+          {
+            std::string tmp = argument.attribute("enum").value();
+            if(tmp.find('.') == std::string::npos)
+            {
+              arg.enum_iface = iface.name;
+              arg.enum_name = tmp;
+            }
+            else
+            {
+              arg.enum_iface = unprefix(tmp.substr(0, tmp.find('.')));
+              arg.enum_name = tmp.substr(tmp.find('.')+1);
+            }
+          }
+
+          arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
+
+          if(arg.type == "new_id")
+            req.ret = arg;
+          req.args.push_back(arg);
+        }
+        iface.requests.push_back(req);
+      }
+
+      for(auto const& event : interface.children("event"))
+      {
+        event_t ev;
+        ev.name = event.attribute("name").value();
+
+        if(event.attribute("since"))
+          ev.since = std::stoi(std::string(event.attribute("since").value()), nullptr, 0);
+        else
+          ev.since = 1;
+
+        if(event.child("description"))
+        {
+          auto description = event.child("description");
+          ev.summary = description.attribute("summary").value();
+          ev.description = description.text().get();
+        }
+
+        for(auto const& argument : event.children("arg"))
+        {
+          argument_t arg;
+          arg.type = argument.attribute("type").value();
+          arg.name = argument.attribute("name").value();
+
+          if(argument.attribute("summary"))
+            arg.summary = argument.attribute("summary").value();
+
+          if(argument.attribute("interface"))
+            arg.interface = unprefix(argument.attribute("interface").value());
+
+          if(argument.attribute("enum"))
+          {
+            std::string tmp = argument.attribute("enum").value();
+            if(tmp.find('.') == std::string::npos)
+            {
+              arg.enum_iface = iface.name;
+              arg.enum_name = tmp;
+            }
+            else
+            {
+              arg.enum_iface = unprefix(tmp.substr(0, tmp.find('.')));
+              arg.enum_name = tmp.substr(tmp.find('.')+1);
+            }
+          }
+
+          arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
+
+          ev.args.push_back(arg);
+        }
+        iface.events.push_back(ev);
+      }
+
+      for(auto const& enumeration : interface.children("enum"))
+      {
+        enumeration_t enu;
+        enu.name = enumeration.attribute("name").value();
+        if(enumeration.child("description"))
+        {
+          auto description = enumeration.child("description");
+          enu.summary = description.attribute("summary").value();
+          enu.description = description.text().get();
+        }
+
+        if(enumeration.attribute("bitfield"))
+        {
+          std::string tmp = enumeration.attribute("bitfield").value();
+          enu.bitfield = (tmp == "true");
+        }
+        else
+          enu.bitfield = false;
+        enu.id = enum_id++;
+        enu.width = 0;
+
+        for(auto entry = enumeration.child("entry"); entry;
+            entry = entry.next_sibling("entry"))
+        {
+          enum_entry_t enum_entry;
+          enum_entry.name = entry.attribute("name").value();
+          if(enum_entry.name == "default"
+             || isdigit(enum_entry.name.at(0)))
+            enum_entry.name.insert(0, 1, '_');
+          enum_entry.value = entry.attribute("value").value();
+
+          if(entry.attribute("summary"))
+            enum_entry.summary = entry.attribute("summary").value();
+
+          auto tmp = static_cast<uint32_t>(std::log2(stol(enum_entry.value, nullptr, 0))) + 1U;
+          if(tmp > enu.width)
+            enu.width = tmp;
+
+          enu.entries.push_back(enum_entry);
+        }
+        iface.enums.push_back(enu);
+      }
+
+      interfaces.push_back(iface);
     }
+  }
 
   std::string hpp_file(extra[extra.size()-2]);
   std::string cpp_file(extra[extra.size()-1]);
@@ -958,7 +958,7 @@ int main(int argc, char *argv[])
   for(auto const& iface : interfaces)
     wayland_hpp << iface.print_interface_header();
   wayland_hpp  << "}" << std::endl
-  << std::endl;
+               << std::endl;
 
   // class declarations
   for(auto const& iface : interfaces)
@@ -995,89 +995,97 @@ int main(int argc, char *argv[])
 
 // set of C++-only keywords not to use as names
 const std::set<std::string> element_t::keywords =
-  {
-   "alignas",
-   "alignof",
-   "and",
-   "and_eq",
-   "asm",
-   "auto",
-   "bitand",
-   "bitor",
-   "bool",
-   "break",
-   "case",
-   "catch",
-   "char",
-   "char16_t",
-   "char32_t",
-   "class",
-   "compl",
-   "const",
-   "constexpr",
-   "const_cast",
-   "continue",
-   "decltype",
-   "default",
-   "delete",
-   "do",
-   "double",
-   "dynamic_cast",
-   "else",
-   "enum",
-   "explicit",
-   "export",
-   "extern",
-   "false",
-   "float",
-   "for",
-   "friend",
-   "goto",
-   "if",
-   "inline",
-   "int",
-   "long",
-   "mutable",
-   "namespace",
-   "new",
-   "noexcept",
-   "not",
-   "not_eq",
-   "nullptr",
-   "operator",
-   "or",
-   "or_eq",
-   "private",
-   "protected",
-   "public",
-   "register",
-   "reinterpret_cast",
-   "return",
-   "short",
-   "signed",
-   "sizeof",
-   "static",
-   "static_assert",
-   "static_cast",
-   "struct",
-   "switch",
-   "template",
-   "this",
-   "thread_local",
-   "throw",
-   "true",
-   "try",
-   "typedef",
-   "typeid",
-   "typename",
-   "union",
-   "unsigned",
-   "using",
-   "virtual",
-   "void",
-   "volatile",
-   "wchar_t",
-   "while",
-   "xor",
-   "xor_eq",
-  };
+{
+  "alignas",
+  "alignof",
+  "and",
+  "and_eq",
+  "asm",
+  "auto",
+  "bitand",
+  "bitor",
+  "bool",
+  "break",
+  "case",
+  "catch",
+  "char",
+  "char8_t",
+  "char16_t",
+  "char32_t",
+  "class",
+  "compl",
+  "concept",
+  "const",
+  "consteval",
+  "constexpr",
+  "constinit",
+  "const_cast",
+  "continue",
+  "co_await",
+  "co_return",
+  "co_yield",
+  "decltype",
+  "default",
+  "delete",
+  "do",
+  "double",
+  "dynamic_cast",
+  "else",
+  "enum",
+  "explicit",
+  "export",
+  "extern",
+  "false",
+  "float",
+  "for",
+  "friend",
+  "goto",
+  "if",
+  "inline",
+  "int",
+  "long",
+  "mutable",
+  "namespace",
+  "new",
+  "noexcept",
+  "not",
+  "not_eq",
+  "nullptr",
+  "operator",
+  "or",
+  "or_eq",
+  "private",
+  "protected",
+  "public",
+  "register",
+  "reinterpret_cast",
+  "requires",
+  "return",
+  "short",
+  "signed",
+  "sizeof",
+  "static",
+  "static_assert",
+  "static_cast",
+  "struct",
+  "switch",
+  "template",
+  "this",
+  "thread_local",
+  "throw",
+  "true",
+  "try",
+  "typedef",
+  "typeid",
+  "typename",
+  "union",
+  "unsigned",
+  "using",
+  "virtual",
+  "void",
+  "volatile",
+  "wchar_t",
+  "while",
+  "xor",
+  "xor_eq",
+};
